package com.fingerprintplay;

import org.apache.commons.io.IOUtils;
import org.mozilla.javascript.*;
import org.mozilla.javascript.ast.AstRoot;
import org.mozilla.javascript.ast.ScriptNode;
import org.nustaq.serialization.FSTConfiguration;
import org.nustaq.serialization.FSTObjectInput;
import org.nustaq.serialization.FSTObjectOutput;

import java.io.*;
import java.lang.reflect.Method;

public class Main {

    static FSTConfiguration gFSTConfiguration;
    static Context gContext;

    public static void main(String[] args) throws Exception
    {
        // input/output filenames (assuming working directory is test)
        String js = "rangy-core.js";
        String ridfst = "rangy-core.js.ridfst";

        // read contents of test JavaScript file
        FileInputStream jsFile = new FileInputStream(js);
        String javascript = IOUtils.toString(jsFile);
        jsFile.close();

        // set up Rhino JavaScript Context
        gContext = Context.enter();
        gContext.setOptimizationLevel(-1); // otherwise won't work on Android :-(
        gContext.initStandardObjects();

        // run test loop
        for (int i = 0; i < 100; i++) {
            // I understand that this object is intended to be re-used... this is a stress
            // test built to demonstrate a problem that occurs very intermittently, even when only single
            // FSTConfiguration is used
            gFSTConfiguration = FSTConfiguration.createAndroidDefaultConfiguration();

            // get the compiled JavaScript object
            Object compiledJavaScript = createCompiledJavaScript(javascript);

            // serialize to a file using FSTObjectOutput
            FileOutputStream fileOutputStream = new FileOutputStream(ridfst);
            FSTObjectOutput out = new FSTObjectOutput(fileOutputStream, gFSTConfiguration);
            out.writeObject(compiledJavaScript);
            out.close(); // required !
            fileOutputStream.close();

            try {
                // deserialize from file using FSTObjectInput - will occasionally fail (because output was corrupt?)
                FileInputStream fileInputStream = new FileInputStream(ridfst);
                FSTObjectInput in = new FSTObjectInput(fileInputStream, gFSTConfiguration);
                Object result = in.readObject();
                in.close(); // required !
                fileInputStream.close();
            } catch (Exception e) {
                System.err.println("deserialize failed: " + e.toString());
            }
        }

        gContext.exit();
    }

    // use Rhino to turn JavaScript string into ready to use representation
    // pre-processing this at build time for Android, to save runtime loading time on device - this is a huge win!
    // thanks for your library to make this an even bigger win than ObjectOutputStream
    static Object createCompiledJavaScript(String javascript)
    {
        Object result = null;

        try {
            CompilerEnvirons compilerEnv = new CompilerEnvirons();
            compilerEnv.initFromContext(gContext);

            ErrorReporter reporter = compilerEnv.getErrorReporter();
            Parser p = new Parser(compilerEnv, reporter);
            AstRoot ast = p.parse(javascript, "string", 1);
            IRFactory irf = new IRFactory(compilerEnv, reporter);
            ScriptNode tree = irf.transformTree(ast);

            // get private createCompiler method via reflection so we can call it
            Method method = gContext.getClass().getDeclaredMethod("createCompiler");
            method.setAccessible(true);
            Evaluator compiler = (Evaluator) method.invoke(gContext);

            result = compiler.compile(compilerEnv, tree, tree.getEncodedSource(), false);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }
}
